using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace GR
{

    public class BBSOrderClass
    {
        public void Reset()
        {
            _targetCPI = 0;
            _targetEURUSDAsk = 0;
            _targetUSDJPYAsk = 0;
            _targetEURJPYBid = 0;
            _actualCPI = 0;
            _actualEURUSDAsk = 0;
            _actualUSDJPYAsk = 0;
            _actualEURJPYBid = 0;
            TradeQTY = 0;
            TradeCompleted = false;
        }
        private double _targetCPI, _targetEURUSDAsk, _targetUSDJPYAsk, _targetEURJPYBid;
        public double TargetCPI
        {
            get { return _targetCPI; }
        }
        public double TargetEURUSDAsk
        {
            get { return _targetEURUSDAsk; }
            set
            {
                _targetEURUSDAsk = value;
                SetTargetCPI();
            }
        }
        public double TargetUSDJPYAsk
        {
            get { return _targetUSDJPYAsk; }
            set
            {
                _targetUSDJPYAsk = value;
                SetTargetCPI();
            }
        }
        public double TargetEURJPYBid
        {
            get { return _targetEURJPYBid; }
            set
            {
                _targetEURJPYBid = value;
                SetTargetCPI();
            }
        }
        private void SetTargetCPI()
        {
            if (_targetEURUSDAsk != 0 && _targetUSDJPYAsk != 0 && _targetEURJPYBid != 0)
                _targetCPI = (_targetEURUSDAsk * _targetUSDJPYAsk * (1 / _targetEURJPYBid)) * 100;
        }
        private double _actualCPI, _actualEURUSDAsk, _actualUSDJPYAsk, _actualEURJPYBid;

        public double ActualCPI
        {
            get { return _actualCPI; }
        }
        public double ActualEURUSDAsk
        {
            get { return _actualEURUSDAsk; }
            set
            {
                _actualEURUSDAsk = value;
                SetActualCPI();
            }
        }
        public double ActualUSDJPYAsk
        {
            get { return _actualUSDJPYAsk; }
            set
            {
                _actualUSDJPYAsk = value;
                SetActualCPI();
            }
        }
        public double ActualEURJPYBid
        {
            get { return _actualEURJPYBid; }
            set
            {
                _actualEURJPYBid = value;
                SetActualCPI();
            }
        }
        private void SetActualCPI()
        {
            if (_actualEURUSDAsk != 0 && _actualUSDJPYAsk != 0 && _actualEURJPYBid != 0)
                _actualCPI = (_actualEURUSDAsk * _actualUSDJPYAsk * (1 / _actualEURJPYBid)) * 100;
        }

        private int _tradeQTY;
        public int TradeQTY
        {
            get { return _tradeQTY; }
            set
            {
                _tradeQTY = value;
                //double qty because we want to detect whether the next
                //transaction is completed
                _EURUSDQTY = (Reverse) ? (-(value) * 2) : -(value);
                _USDJPYQTY = (Reverse) ? (-(value) * 2) : -(value);
                _EURJPYQTY = (Reverse) ? (-(value) * 2) : -(value);
            }
        }

        private int _EURUSDQTY, _USDJPYQTY, _EURJPYQTY;
        public int EURUSDQTY
        {
            get { return _EURUSDQTY; }
            set
            {
                _EURUSDQTY += value;
                CheckCompleted();
            }
        }
        public int USDJPYQTY
        {
            get { return _USDJPYQTY; }
            set
            {
                _USDJPYQTY += value;
                CheckCompleted();
            }
        }
        public int EURJPYQTY
        {
            get { return _EURJPYQTY; }
            set
            {
                _EURJPYQTY += value;
                CheckCompleted();
            }
        }
        private void CheckCompleted()
        {
            if (_EURUSDQTY == 0 && _USDJPYQTY == 0 && _EURJPYQTY == 0)
                TradeCompleted = true;
        }
        public bool Reverse = false;
        private bool _tradeCompleted = false;
        public bool TradeCompleted
        {
            get { return _tradeCompleted; }
            set
            {
                _tradeCompleted = value;
                if (value)
                    LogTrade();
            }
        }
        private void LogTrade()
        {
            try
            {
                string sPathName = System.Windows.Forms.Application.StartupPath.Remove(System.Windows.Forms.Application.StartupPath.IndexOf("bin")) + @"TradeLogs\";
                string sLogTime = DateTime.Now.ToShortDateString().ToString() + " " + DateTime.Now.ToLongTimeString().ToString();
                StreamWriter sw = new StreamWriter(sPathName + DateTime.Now.ToShortDateString().Replace('/','_') + ".txt", true);
                sw.WriteLine(sLogTime + ": BBS");
                sw.WriteLine("TargetCPI: " + _targetCPI.ToString() + ", TargetEURUSDAsk: " + _targetEURUSDAsk.ToString() + ", TargetUSDJPYAsk: " + _targetUSDJPYAsk.ToString() + ", TargetEURJPYBid: " + _targetEURJPYBid.ToString());
                sw.WriteLine("ActualCPI: " + _actualCPI.ToString() + ", ActualEURUSDAsk: " + _actualEURUSDAsk.ToString() + ", ActualUSDJPYAsk: " + _actualUSDJPYAsk.ToString() + ", ActualEURJPYBid: " + _actualEURJPYBid.ToString());
                sw.WriteLine("");
                sw.Flush();
                sw.Close();
            }
            catch (System.Exception er)
            {
             //   System.Windows.Forms.MessageBox.Show("Logging error: " + er.Message);
            }
        }
    }
}
