using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Reflection;
using MBTCOMLib;
using MBTORDERSLib;
using MBTQUOTELib;

namespace GR
{

	/// <summary>
	/// Summary description for MBTInterface.
	/// </summary>
	public class MBTInterface
	{
		public MBTCOMLib.IMbtComMgr			m_ComMgr;
		MBTORDERSLib.IMbtOrderClient	m_OrderClient;
		MBTORDERSLib.IMbtAccounts		m_Accounts;	// all of 'em
		MBTORDERSLib.IMbtAccount		m_Account;	// singular
		MBTORDERSLib.MbtAccount			m_DefaultAccount;	// Direct class vs interface???
        public MBTQUOTELib.IMbtQuotes m_Quotes;

        MBTQuoteCB qn;

		public MBTInterface()
		{
			m_ComMgr	= null;
          
		}

		public void Login(int memberId, string username, string password )
		{
			m_ComMgr = new MBTCOMLib.MbtComMgrClass();
//			m_ComMgr.EnableSplash(false); // uncomment to disable the splash screen


            m_ComMgr.OrderClient.OnHistoryAdded += new _IMbtOrderClientEvents_OnHistoryAddedEventHandler(OrderClient_OnHistoryAdded);
			m_OrderClient = m_ComMgr.OrderClient;

			m_OrderClient.OnDemandMode = false;
//			m_OrderClient.SilentMode = true; // uncomment to disable dialogs

			if (m_ComMgr.DoLogin(425, username, password, "") )
			{
				MessageBox.Show("DoLogin Succeeded - Please wait for all accounts to load.");
			}

			// Spin a message loop so the OrderClient object can complete its login
			MSG msg = new MSG();
			int iMaxWait = 10000;  // Timeout for login (milliseconds)
			uint dwStart = User32.GetTickCount();
			while( User32.GetMessage( ref msg, User32.NULL, (uint)0, (uint)0 ) != 0)
			{
				User32.TranslateMessage( ref msg );
				User32.DispatchMessage( ref msg );

				if( User32.GetTickCount() > dwStart + iMaxWait )
				{
					throw new Exception("GotInitialLogonSucceed() failed !");
				}
				if( m_OrderClient.GotInitialLogonSucceed() )
				{
					uint dwEnd = User32.GetTickCount();
					break;
				}
			}

			m_Accounts = m_OrderClient.Accounts;
			int iCount = m_Accounts.Count;
			Console.WriteLine("");
			Console.WriteLine("We have " + iCount.ToString() + " account(s) ----------------");
			for( int i = 0; i < iCount; i++ )
			{
				m_Account = m_Accounts[i];	// Maybe this is the default [] operator???
				m_Account.Load();
			}

			m_DefaultAccount = m_Accounts.DefaultAccount;
			if( m_DefaultAccount != null )
			{
			}
			else
				throw( new Exception("Could not get DefaultAccount !") );
        }

        void OrderClient_OnHistoryAdded(MbtOrderHistory pHist)
        {
            System.Windows.Forms.MessageBox.Show(pHist.Event);
        }
 
        #region GetQuote
        
        public MBTQuoteCB GetQuote(string _symbol)
        {
			if (m_ComMgr == null )
			{
				throw new Exception("You must login first");
			}
			m_Quotes = m_ComMgr.Quotes;
               
			MSG msg = new MSG();
			MBTQuoteCB qn = new MBTQuoteCB(m_Quotes);
            qn.Price += new MBTQuoteCB.PriceHandler(OnQuoteUpdate);
            qn.DoAdvise("USD/JPY");
            
            if (qn.DoAdvise(_symbol))
			{
				int iMaxCBs = 9;
				int iMaxTime = 5000;
				uint dwStart = User32.GetTickCount();
				while( User32.GetMessage( ref msg, User32.NULL, 0, 0 ) != 0)
				{
					User32.TranslateMessage( ref msg );
					User32.DispatchMessage( ref msg );
					if( User32.GetTickCount() > dwStart + iMaxTime ) // set a timeout
					{
						break;
					}
					if( qn.GotQuote() )  // if we have a callback...
					{
						qn.SetQuoteWaiting();
                        iMaxCBs--;
                        if( 1 > iMaxCBs )
                        {
                            break;
                        }
                    }
				}
			}
			else
				throw new Exception("Could not DoAdvise() !" );

            return qn;
        }
        #endregion
        /*
        public MBTQuoteCB GetQuotes(string _symbol)
		{
			if (m_ComMgr == null )
			{
				throw new Exception("You must login first");
			}
			m_Quotes = m_ComMgr.Quotes;
            
			Console.WriteLine("");
			Console.WriteLine("Quotes ------------------------------");
			Console.WriteLine(" Remote Address  : " + m_Quotes.RemoteAddress );
			Console.WriteLine(" Remote Port     : " + m_Quotes.RemotePort.ToString() );
			Console.WriteLine(" Username        : " + m_Quotes.UserName );
            
			// Set up our callback object and do the request.
			// Intentional stackframe so our qn goes out of scope and releases
			// everything it holds a reference to BEFORE main() ends
             
			MSG msg = new MSG();
			qn = new MBTQuoteCB(m_Quotes);
            qn.Price += new MBTQuoteCB.PriceHandler(OnQuoteUpdate);
			Console.WriteLine("Advising symbol...\n");
            if (qn.DoAdvise(_symbol))
			{
				int iMaxCBs = 9;    // callbacks limit
				int iMaxTime = 5000;       // milliseconds limit
				Console.WriteLine("");
				Console.WriteLine("Process callbacks until {0}ms timeout or {1} callbacks:", iMaxTime, iMaxCBs);
				// Hang out pump message loop subject to a timeout and a number of callbacks
				// to get back a quote
				uint dwStart = User32.GetTickCount();
				while( User32.GetMessage( ref msg, User32.NULL, 0, 0 ) != 0)
				{
					User32.TranslateMessage( ref msg );
					User32.DispatchMessage( ref msg );
                    
					if( User32.GetTickCount() > dwStart + iMaxTime ) // set a timeout
					{
						Console.WriteLine("Timeout hit\n");
						break;
					}
                    
					if( qn.GotQuote() )  // if we have a callback...
					{
						// ...process it, reset our flag, and check max callback counter.
						CurrentQuote qt = qn.GetLastQuote();
						Console.WriteLine(
							String.Format("{0}.  {1}  bid {2}  ask {3}  last {4}  vol {5}  tick {6}",
					//	printf("%d.  %s  bid %g  ask %g  last %g  vol %d  tick %s\n",
							iMaxCBs, qt.sTime, qt.dBid, qt.dAsk, qt.dLast, qt.lVol, qt.eTick));
						qn.SetQuoteWaiting();
                        
                            iMaxCBs--;
                            if( 1 > iMaxCBs )
                            {
                                Console.WriteLine("Max callbacks received\n");
                                break;
                            }
                    }
				}
			}
			else
				throw new Exception("Could not DoAdvise() !" );

            return qn;
		}
        */
        void OnQuoteUpdate(CurrentQuote cq, EventArgs e)
        {
            System.Windows.Forms.MessageBox.Show(cq.dLast.ToString());
        }

		public void SubmitOrder()
		{
			Console.WriteLine("Now submitting an order; m_OrderClient.Submit()");
			int lBuySell = MBConst.VALUE_BUY;
			int	lVol = 100;
			string sSym = "MSFT";
			// Arbitrary price. Should be filled in with the
			//  last price for all Market and Stop orders
			double dPrice = 25.75;
			int	lTIF = MBConst.VALUE_DAY;
			int lOrdType = MBConst.VALUE_LIMIT;
			int lVolType = MBConst.VALUE_NORMAL;
			string sRoute = "MBTX";
			string bstrRetMsg = null; // NULL works, but still need to free below ::SysAllocString(L"");
			System.DateTime dDte = new System.DateTime(0);

			string sOrder = String.Format("Buy {0} {1} at {2:c} on {3}", lVol, sSym, dPrice, sRoute);
			Console.WriteLine(sOrder);
            
			bool bSubmit = m_OrderClient.Submit(
				lBuySell,
				lVol,
				sSym,
				dPrice,
				0.0,
				lTIF,
				0,
				lOrdType,
				lVolType,
				0,
				m_DefaultAccount,
				sRoute,
				"", 0, 0,
				dDte, dDte,
				0, 0, 0, 0,0,
				ref bstrRetMsg
			);

			if( bSubmit )
			{
				Console.WriteLine(
					String.Format("Order submission successful! bstrRetMsg = [{0}]", bstrRetMsg)); // wide string!
			}
			else
			{
				Console.WriteLine(
					String.Format("Order submission failed! bstrRetMsg = [{0}]", bstrRetMsg)); // wide string!
			}
		}
	}
}
