using System;
using MBTCOMLib;
using MBTORDERSLib;
using MBTQUOTELib;


namespace GR
{
	public class CurrentQuote // stuffed on callback, used by main()
	{
        

		public string sTime;
        public double dLast;
		public double dBid;
		public double dAsk;
		public int	  lVol;
		public string eTick;  // "v" or "^"
		public DateTime dDte;
	};

	/// <summary>
	/// Summary description for MBTQuoteCB.
	/// </summary>
	public class MBTQuoteCB : IMbtQuotesNotify 
	{
		public IMbtQuotes 			m_Quotes;
		private bool				m_GotQuote;
		public CurrentQuote		m_LastQuote = new CurrentQuote();

        public event PriceHandler Price;
        public EventArgs e = null;
        public delegate void PriceHandler(CurrentQuote cq, EventArgs e);
        
		public MBTQuoteCB(MBTQUOTELib.IMbtQuotes quotes)
		{
			m_Quotes = quotes;
			/*
			  EnableAutomation();      // Enable OLE automation
			  LPDISPATCH pDisp = GetIDispatch(FALSE); // No addref here
			  m_pQNotify = pDisp;      // Automatic QI by smart pointer
			*/
		}

		~MBTQuoteCB()
		{
			//Stop();
		}

		public void Stop()
		{
			m_Quotes.UnadviseAll( /*(IMbtQuotesNotify)*/this );
		}

		public bool DoAdvise(string symbol)
		{
            
			try
			{
				m_GotQuote = false;
				m_Quotes.AdviseSymbol(this, symbol, (int)enumQuoteServiceFlags.qsfLevelOne);
				return true;
			}
			catch( Exception e )
			{
				Console.WriteLine("Could not AdviseSymbol {0} - {1}!\n", symbol, e.Message);
				return false;
			}
		}

		public bool GotQuote()
		{
			return m_GotQuote;
		}

		public void SetQuoteWaiting()
		{
			m_GotQuote = false;
		}

		public CurrentQuote GetLastQuote()
		{
			return m_LastQuote;
		}
		#region IMbtQuotesNotify Members

		void MBTQUOTELib.IMbtQuotesNotify.OnOptionsData(ref OPTIONSRECORD pRec)
		{
			Console.WriteLine("Got Quote");
		}

		void MBTQUOTELib.IMbtQuotesNotify.OnTSData(ref TSRECORD pRec)
		{
			Console.WriteLine("Got Quote");
		}

		void MBTQUOTELib.IMbtQuotesNotify.OnQuoteData(ref QUOTERECORD pQuote)
		{
			// In a real app, this might just call into another object or something... for this
			// example, we'll just copy pertinent quote data to our local quote structure and
			// set our flag so main() can interrogate us and extract the data we stuffed.
			// The first record may or may not contain price data... it MAY only contain
			// fundamental information.
			m_LastQuote.dAsk	= pQuote.dAsk;
			m_LastQuote.dBid	= pQuote.dBid;
			m_LastQuote.dLast	= pQuote.dLast;
			m_LastQuote.eTick	= (pQuote.tick == enumTick.tickDown ? "v" : "^");
			m_LastQuote.lVol	= pQuote.lVolume;
			m_LastQuote.dDte  = pQuote.UTCDateTime;
			this.m_GotQuote		= true;
            if (Price != null)
            {
                Price(m_LastQuote, e);
            }
		}

		void MBTQUOTELib.IMbtQuotesNotify.OnLevel2Data(ref LEVEL2RECORD pRec)
		{
			Console.WriteLine("Got Quote");
		}

		#endregion

	}
}
