using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Xml;
using System.IO;
using System.Windows.Forms;
using MBTCOMLib;
using MBTORDERSLib;
using MBTQUOTELib;

namespace GR
{

    public partial class frmMain : Form, MBTQUOTELib.IMbtQuotesNotify
    {
        frmSetup fSet;
        public event PriceHandler Price;
        public EventArgs e = null;
        public delegate void PriceHandler(EventArgs e);
        Random rand;
        TradeState _CurrentState;
        BBSOrderClass BBSOrder = new BBSOrderClass();
        SSBOrderClass SSBOrder = new SSBOrderClass();
        public TradeState CurrentState
        {
            get { return _CurrentState; }
            set 
            {
                _CurrentState = value;
                toollblStatus.Text = _CurrentState.ToString();
            }
        }

        string[] symbols = { "EUR/USD", "USD/JPY", "EUR/JPY" };
        DateTime dQuoteDate;

        #region Currency/Trade variables
        //trade
        int iQuantity = 10000;
        bool DoReverse = false, CanTrade = false;
        //cpi
        double ExitCPI = 100;
        bool SeekExit = false;
        double _BBScpi = 0, _SSBcpi = 0;
        double BBSThreshold = 99.98, SSBThreshold = 100.02;
        bool ValidBBS = false, ValidSSB = false;
        double BBScpi
        {
            get { return _BBScpi; }
            set
            {
                _BBScpi = value;
                if (value <= BBSThreshold)
                    ValidBBS = true;
                else
                    ValidBBS = false;
            }
        }
        double SSBcpi
        {
            get { return _SSBcpi; }
            set
            {
                _SSBcpi = value;
                if (value >= SSBThreshold)
                    ValidSSB = true;
                else
                    ValidSSB = false;
            }
        }
        //currencies
        double _EURUSDBid, _EURUSDAsk, _USDJPYBid, _USDJPYAsk, _EURJPYBid, _EURJPYAsk;
        double EURUSDBid1, EURUSDAsk1, USDJPYBid1, USDJPYAsk1, EURJPYBid1, EURJPYAsk1;
        double EURUSDBid2, EURUSDAsk2, USDJPYBid2, USDJPYAsk2, EURJPYBid2, EURJPYAsk2;
        double EURUSDBid3, EURUSDAsk3, USDJPYBid3, USDJPYAsk3, EURJPYBid3, EURJPYAsk3;
        double EURUSDBid4, EURUSDAsk4, USDJPYBid4, USDJPYAsk4, EURJPYBid4, EURJPYAsk4;
        double EURUSDBid
        {
            get { return _EURUSDBid; }
            set 
            {
                _EURUSDBid = value;
                lblEUBid.Text = value.ToString();
            }
        }
        double EURUSDAsk
        {
            get { return _EURUSDAsk; }
            set 
            {
                _EURUSDAsk = value;
                lblEUAsk.Text = value.ToString();
                if (Price != null)
                    Price(e);
            }
        }
        double USDJPYBid
        {
            get { return _USDJPYBid; }
            set 
            {
                _USDJPYBid = value;
                lblUJBid.Text = value.ToString();
            }
        }
        double USDJPYAsk
        {
            get { return _USDJPYAsk; }
            set 
            {
                _USDJPYAsk = value;
                lblUJAsk.Text = value.ToString();
                if (Price != null)
                    Price(e);
            }
        }
        double EURJPYBid
        {
            get { return _EURJPYBid; }
            set 
            {
                _EURJPYBid = value;
                lblEJBid.Text = value.ToString();
            }
        }
        double EURJPYAsk
        {
            get { return _EURJPYAsk; }
            set 
            {
                _EURJPYAsk = value;
                lblEJAsk.Text = value.ToString();
                if (Price != null)
                    Price(e);
            }
        }
        //trade state
        public enum TradeState
        {
            FlatAll = 0,
            BBSOpening = 1,
            BBSOpenComplete = 2,
            BBSClosing = 3,
            BBSCloseCompleted = 4,
            BBSReversing = 5,
            BBSReversalCompleted = 6,
            SSBOpening = 7,
            SSBOpenComplete = 8,
            SSBClosing = 9,
            SSBCloseCompleted = 10,
            SSBReversing = 11,
            SSBReversalCompleted = 12,
            ClosingAll = 13
        }

        #endregion

        private const int IDS_GOTINITLOGONSUCCEEDFAILED = 4;
        private const int IDS_NODEFAULTACCOUNT = 5;
        private const int IDS_DOADVISEFAILED = 6;

        MBTCOMLib.IMbtComMgr m_ComMgr;
        MBTORDERSLib.IMbtOrderClient m_OrderClient;
        MBTORDERSLib.IMbtAccounts m_Accounts;
        MBTORDERSLib.IMbtAccount m_Account;	
        MBTORDERSLib.MbtAccount m_DefaultAccount;	
        MBTQUOTELib.IMbtQuotes m_Quotes;

        public frmMain()
        {
            InitializeComponent();
            this.Price += new PriceHandler(OnPriceChanged);
            this.FormClosing += new FormClosingEventHandler(frmMain_FormClosing);
            m_ComMgr = null;
            fSet = new frmSetup();
            fSet.LoginCalled += new frmSetup.LoginHandler(fSet_LoginCalled);
            fSet.AutoLogin();
            if (!fSet.bAutoLogin)
                fSet.Show(this);
            if(Convert.ToBoolean(System.Configuration.ConfigurationSettings.AppSettings["savestate"]))
                CurrentState = (TradeState) Convert.ToInt32(System.Configuration.ConfigurationSettings.AppSettings["state"]);
            else
                CurrentState = TradeState.FlatAll;
        }
        
        #region CPI Calc
        void OnPriceChanged(EventArgs e)
        {
            if (CurrentState == TradeState.ClosingAll && m_OrderClient.Positions.OpenCount == 0)
                CurrentState = TradeState.FlatAll;
            //update cpi
            if (EURUSDAsk != 0 && USDJPYAsk != 0 && EURJPYBid != 0)
                BBScpi = Math.Round((EURUSDAsk * USDJPYAsk * (1 / EURJPYBid)) * 100, 4);
            else
                BBScpi = 100;
            if (EURUSDBid != 0 && USDJPYBid != 0 && EURJPYAsk != 0)
                SSBcpi = Math.Round((EURUSDBid * USDJPYBid * (1 / EURJPYAsk)) * 100, 4);
            else
                SSBcpi = 0;
            lblBBScpi.Text = BBScpi.ToString();
            lblSSBcpi.Text = SSBcpi.ToString();

            LogData();

            if (SeekExit)
                CloseAllTrades();
            if (!CanTrade)
                return;
            if (ckWaitExtremes.Checked && (!ValidBBS || !ValidSSB)) //never allows entry unless signal on both sides
                return;
            if (!ValidBBS && !ValidSSB)
                return;
            if (CurrentState == TradeState.FlatAll && ValidBBS && ValidSSB)
            {
                //randonmly choose which to enter
                int r = 0;
                rand = new Random();
                r = rand.Next(1,3);
                if (r == 1)//bbs
                {
                    CurrentState = TradeState.BBSOpening;
                    EnterBBS();
                }
                else //ssb
                {
                    CurrentState = TradeState.SSBOpening;
                    EnterSSB();
                }
            }
               
            else if (CurrentState == TradeState.FlatAll && ValidBBS && !ValidSSB)
            {
                //bbs
                CurrentState = TradeState.BBSOpening;
                EnterBBS();
            }
            else if (CurrentState == TradeState.FlatAll && ValidSSB && !ValidBBS)
            {
                //ssb
                CurrentState = TradeState.SSBOpening;
                EnterSSB();
            }
                 
            else if ((CurrentState == TradeState.BBSOpenComplete || CurrentState == TradeState.SSBReversalCompleted) && !DoReverse && ValidSSB )
            {
                //ssb
                CurrentState = TradeState.BBSClosing ;
                EnterSSB();
            }
            else if ((CurrentState == TradeState.SSBOpenComplete || CurrentState == TradeState.BBSReversalCompleted)&& !DoReverse && ValidBBS )
            {
                //bbs
                CurrentState = TradeState.SSBClosing;
                EnterBBS();
            }
            else if ((CurrentState == TradeState.BBSOpenComplete || CurrentState == TradeState.SSBReversalCompleted) && DoReverse && ValidSSB)
            {
                //bbs
                CurrentState = TradeState.BBSReversing;
                EnterSSB();
            }
            else if ((CurrentState == TradeState.SSBOpenComplete || CurrentState == TradeState.BBSReversalCompleted) && DoReverse && ValidBBS)
            {
                //bbs
                CurrentState = TradeState.SSBReversing;
                EnterBBS();
            }
        }
        public void EnterBBS()
        {
            BBSOrder.Reset();
            BBSOrder.Reverse = (CurrentState == TradeState.SSBReversing)?true:false;
            BBSOrder.TradeQTY = iQuantity;
            BBSOrder.TargetEURUSDAsk = EURUSDAsk;
            BBSOrder.TargetUSDJPYAsk = USDJPYAsk;
            BBSOrder.TargetEURJPYBid = EURJPYBid;
            SendBBS();
        }
        public void EnterSSB()
        {
            SSBOrder.Reset();
            SSBOrder.Reverse = (CurrentState == TradeState.BBSReversing) ? true : false;
            SSBOrder.TradeQTY = iQuantity;
            SSBOrder.TargetEURUSDBid = EURUSDBid;
            SSBOrder.TargetUSDJPYBid = USDJPYBid;
            SSBOrder.TargetEURJPYAsk = EURJPYAsk;
            SendSSB();
        }
        #endregion

        #region Genral
        private void Form1_Load(object sender, EventArgs e)
        {
        }

        private void btnStarted_Click(object sender, EventArgs e)
        {
            CanTrade = !CanTrade;
            if (CanTrade)
                btnStarted.Text = "Stop";
            else
                btnStarted.Text = "Start";
            OnPriceChanged(EventArgs.Empty);
        }
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }
        private void toolStripFile_Click(object sender, EventArgs e)
        {
            fSet.Show();
        }
        private void nudOrderSize_ValueChanged(object sender, EventArgs e)
        {
            if (nudOrderSize.Value != iQuantity)
                btnSetOrderSize.Enabled = true;
            else
                btnSetOrderSize.Enabled = false;
        }

        private void btnSetOrderSize_Click(object sender, EventArgs e)
        {
            iQuantity = (int)nudOrderSize.Value;
            btnSetOrderSize.Enabled = false;
        }

        private void nudBBS_ValueChanged(object sender, EventArgs e)
        {
            if ((double)nudBBS.Value != BBSThreshold || (double)nudSSB.Value != SSBThreshold)
                btnSetEntries.Enabled = true;
            else
                btnSetEntries.Enabled = false;
        }

        private void nudSSB_ValueChanged(object sender, EventArgs e)
        {
            if ((double)nudBBS.Value != BBSThreshold || (double)nudSSB.Value != SSBThreshold)
                btnSetEntries.Enabled = true;
            else
                btnSetEntries.Enabled = false;
        }

        private void btnSetEntries_Click(object sender, EventArgs e)
        {
            BBSThreshold = (double)nudBBS.Value;
            SSBThreshold = (double)nudSSB.Value;
            btnSetEntries.Enabled = false;
            OnPriceChanged(EventArgs.Empty);
        }
        private void nudSeekExit_ValueChanged(object sender, EventArgs e)
        {
            if (SeekExit && (double)nudSeekExit.Value != ExitCPI)
                btnSetExit.Enabled = true;
            else if (SeekExit)
                btnSetExit.Enabled = false;
        }

        private void btnSetExit_Click(object sender, EventArgs e)
        {
            SeekExit = true;
            ExitCPI = (double)nudSeekExit.Value;
            btnSetExit.Enabled = false;
            btnCancelExit.Enabled = true;
            OnPriceChanged(EventArgs.Empty);
        }
        private void btnCancelExit_Click(object sender, EventArgs e)
        {
            SeekExit = false;
            btnSetExit.Enabled = true;
            btnCancelExit.Enabled = false;
        }
        private void ckReverse_CheckedChanged(object sender, EventArgs e)
        {
            DoReverse = ckReverse.Checked;
        }

        void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            DialogResult dr = MessageBox.Show("Save current trade state?", "", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2);
            if (dr == DialogResult.Cancel)
            {
                e.Cancel = true;
                return;
            }
            else if (dr == DialogResult.Yes)
            {
                replaceConfigSettings("savestate", "true");
                replaceConfigSettings("state", CurrentState.GetHashCode().ToString());
            }
            else
            {
                replaceConfigSettings("savestate", "false");
            }
            m_Quotes.UnadviseAll(this);
        }
        public void replaceConfigSettings(string key, string val)
        {
            XmlDocument xDoc = new XmlDocument();
            string sFileName = Application.StartupPath + @"\GRArb.exe.config";
            try
            {
                // load the configuration file
                xDoc.Load(sFileName);
                // find the node of interest containing the key using XPATH
                XmlNode theNode = xDoc.SelectSingleNode(@"/configuration/appSettings/add[@key = '" + key + "\']");
                // Set the new value for the node
                if (theNode != null)
                    theNode.Attributes["value"].Value = val;
                // lop off file prefix of the filename if it exists
                if (sFileName.StartsWith("file:///"))
                    sFileName = sFileName.Remove(0, 8);
                // save the new configuration settings
                xDoc.Save(sFileName);
            }
            catch (Exception ex)
            {
              //  System.Windows.Forms.MessageBox.Show("replaceConfigSettings()" + ex.Message);
                xDoc = null;
            }
        }
        private void btnResetState_Click(object sender, EventArgs e)
        {
            this.CurrentState = TradeState.FlatAll;
        }
        #endregion

        #region Login

        void fSet_LoginCalled(Button btn, EventArgs e)
        {
            m_ComMgr = new MBTCOMLib.MbtComMgrClass();
            //m_ComMgr.EnableSplash(false); // uncomment to disable the splash screen

            m_ComMgr.OrderClient.OnHistoryAdded += new _IMbtOrderClientEvents_OnHistoryAddedEventHandler(OrderClient_OnHistoryAdded);
            m_OrderClient = m_ComMgr.OrderClient;

            m_OrderClient.OnDemandMode = false;
            //m_OrderClient.SilentMode = true; // uncomment to disable dialogs

            if (m_ComMgr.DoLogin(425, fSet.txtUser.Text, fSet.txtPassword.Text, ""))
            {
                fSet.Hide();
                fSet.btnLogin.Enabled = false;
            }
            // Spin a message loop so the OrderClient object can complete its login
            MSG msg = new MSG();
            int iMaxWait = 10000;  // Timeout for login (milliseconds)
            uint dwStart = User32.GetTickCount();
            while (User32.GetMessage(ref msg, User32.NULL, (uint)0, (uint)0) != 0)
            {
                User32.TranslateMessage(ref msg);
                User32.DispatchMessage(ref msg);

                if (User32.GetTickCount() > dwStart + iMaxWait)
                {
                    throw new Exception("GotInitialLogonSucceed() failed !");
                }
                if (m_OrderClient.GotInitialLogonSucceed())
                {
                    uint dwEnd = User32.GetTickCount();
                    break;
                }
            }

            m_Accounts = m_OrderClient.Accounts;
            int iCount = m_Accounts.Count;
 
            for (int i = 0; i < iCount; i++)
            {
                m_Account = m_Accounts[i];	// Maybe this is the default [] operator???
                m_Account.Load();
            }

            m_DefaultAccount = m_Accounts.DefaultAccount;
            if (m_DefaultAccount != null)
            {
            }
            else
                throw (new Exception("Could not get DefaultAccount !"));

            SubscribeSymbols();
        }
        #endregion

        #region Quotes

        void MBTQUOTELib.IMbtQuotesNotify.OnQuoteData(ref MBTQUOTELib.QUOTERECORD pQuote)
        {
            dQuoteDate = pQuote.UTCDateTime.ToLocalTime();
            if (pQuote.dAsk == 0 || pQuote.dBid == 0)
                return;
            //TODO: Add tick filter of 30 ticks or so
            switch (pQuote.bstrSymbol)
            {
                case "EUR/USD":
                    if (EURUSDBid == pQuote.dBid && EURUSDAsk == pQuote.dAsk)
                        break;
                    EURUSDBid = pQuote.dBid;
                    EURUSDAsk = pQuote.dAsk;
                    break;
                case "USD/JPY":
                    if (USDJPYBid == pQuote.dBid && USDJPYAsk == pQuote.dAsk)
                        break;
                    USDJPYBid = pQuote.dBid;
                    USDJPYAsk = pQuote.dAsk;
                    break;
                case "EUR/JPY":
                    if (EURJPYBid == pQuote.dBid && EURJPYAsk == pQuote.dAsk)
                        break;
                    EURJPYBid = pQuote.dBid;
                    EURJPYAsk = pQuote.dAsk;
                    break;
            }
          
        }
        public void SubscribeSymbols()
        {
            m_Quotes = m_ComMgr.Quotes;

            
            for (int i = 0; i < symbols.Length; i++)
            {
                try
                {
                    m_Quotes.AdviseSymbol(this, symbols[i], (int)enumQuoteServiceFlags.qsfLevelOne);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Could not AdviseSymbol "+symbols[i]+" - "+e.Message+"!\n");
                }
            }
        }
        void MBTQUOTELib.IMbtQuotesNotify.OnLevel2Data(ref MBTQUOTELib.LEVEL2RECORD pRec)
        {//not used
        }
        void MBTQUOTELib.IMbtQuotesNotify.OnOptionsData(ref MBTQUOTELib.OPTIONSRECORD pRec)
        {//not used
        }
        void MBTQUOTELib.IMbtQuotesNotify.OnTSData(ref MBTQUOTELib.TSRECORD pRec)
        {//not used
        }

        #endregion

        #region Orders

        void OrderClient_OnHistoryAdded(MbtOrderHistory pHist)
        {
            if (pHist.Event.ToLower() == "executed")
            {
               
                switch (CurrentState)
                {
                    case TradeState.BBSOpening:
                    case TradeState.SSBClosing:
                    case TradeState.SSBReversing:
                        switch (pHist.Symbol)
                        {
                            case "EUR/USD":
                                BBSOrder.ActualEURUSDAsk = pHist.Price;
                                BBSOrder.EURUSDQTY = pHist.Quantity;
                                break;
                            case "USD/JPY":
                                BBSOrder.ActualUSDJPYAsk = pHist.Price;
                                BBSOrder.USDJPYQTY = pHist.Quantity;
                                break;
                            case "EUR/JPY":
                                BBSOrder.ActualEURJPYBid = pHist.Price;
                                BBSOrder.EURJPYQTY = pHist.Quantity;
                                break;
                        }
                        if (BBSOrder.TradeCompleted)
                        {
                            if(CurrentState == TradeState.BBSOpening)
                                CurrentState = TradeState.BBSOpenComplete;
                            else if (CurrentState == TradeState.SSBReversing)
                                CurrentState = TradeState.SSBReversalCompleted;
                            else
                                //CurrentState = TradeState.SSBCloseCompleted;
                                CurrentState = TradeState.FlatAll;
                            OnPriceChanged(EventArgs.Empty);
                        }
                        break;
                    case TradeState.SSBOpening:
                    case TradeState.BBSClosing:
                    case TradeState.BBSReversing:
                        switch (pHist.Symbol)
                        {
                            case "EUR/USD":
                                SSBOrder.ActualEURUSDBid = pHist.Price;
                                SSBOrder.EURUSDQTY = pHist.Quantity;
                                break;
                            case "USD/JPY":
                                SSBOrder.ActualUSDJPYBid = pHist.Price;
                                SSBOrder.USDJPYQTY = pHist.Quantity;
                                break;
                            case "EUR/JPY":
                                SSBOrder.ActualEURJPYAsk = pHist.Price;
                                SSBOrder.EURJPYQTY = pHist.Quantity;
                                break;
                        }
                        if (SSBOrder.TradeCompleted)
                        {
                            if (CurrentState == TradeState.SSBOpening)
                                CurrentState = TradeState.SSBOpenComplete;
                            else if (CurrentState == TradeState.BBSReversing)
                                CurrentState = TradeState.BBSReversalCompleted;
                            else
                                //CurrentState = TradeState.BBSCloseCompleted;
                                CurrentState = TradeState.FlatAll;
                            OnPriceChanged(EventArgs.Empty);
                        }
                        break;
                }
            }
        }
        bool SendBBS()
        {
            int iSize = 0;
            if (CurrentState == TradeState.SSBReversing && DoReverse)
                iSize = iQuantity * 2;
            else
                iSize = iQuantity;
            return (SendOrder("EUR/USD", iSize, MBConst.VALUE_BUY, EURUSDAsk) &
            SendOrder("USD/JPY", iSize, MBConst.VALUE_BUY, USDJPYAsk) &
            SendOrder("EUR/JPY", iSize, MBConst.VALUE_SELL, EURJPYBid));
        }
        bool SendSSB()
        {
            int iSize = 0;
            if (CurrentState == TradeState.BBSReversing && DoReverse)
                iSize = iQuantity * 2;
            else
                iSize = iQuantity;
            return (SendOrder("EUR/USD", iSize, MBConst.VALUE_SELL, EURUSDBid) &
            SendOrder("USD/JPY", iSize, MBConst.VALUE_SELL, USDJPYBid) &
            SendOrder("EUR/JPY", iSize, MBConst.VALUE_BUY, EURUSDAsk));
        }
        bool SendOrder(string _sym, int quantity, int direction, double _price)
        {
            string bstrRetMsg = null; // NULL works, but still need to free below ::SysAllocString(L"");
            System.DateTime dDte = new System.DateTime(0);

           bool bSubmit = m_OrderClient.Submit(
                direction,
                quantity,
                _sym,
                _price,
                0.0,
                MBConst.VALUE_GTC,
                0,
                MBConst.VALUE_MARKET,
                MBConst.VALUE_NORMAL,
                0,
                m_DefaultAccount,
                "MBTX",
                "", 0, 0,
                dDte, dDte,
                0, 0, 0, 0, 0,
                ref bstrRetMsg
            );
           return bSubmit;
       }
  
        void CloseAllTrades()
        {
            if (m_OrderClient.Positions.OpenCount > 0)
            {
                if ((rbUseSSB.Checked && SSBcpi >= ExitCPI) || (rbUseBBS.Checked && BBScpi <= ExitCPI))
                {
                    SeekExit = false;
                    btnCancelExit.Enabled = false;
                    btnSetExit.Enabled = true;
                    CurrentState = TradeState.ClosingAll;
                    m_OrderClient.Positions.CloseAll(m_DefaultAccount);
                }
                //need to add new trade status type for exiting and send bbs/ssb
            }
        }
        #endregion

        #region Logging
        private void LogData()
        {
            try
            {
                string sPathName = Application.StartupPath.Remove(Application.StartupPath.IndexOf("bin")) + @"Data\";
                StreamWriter sw = new StreamWriter(sPathName + dQuoteDate.ToShortDateString().Replace('/', '_') + ".txt", true);
                sw.WriteLine(dQuoteDate.ToShortDateString() + ", " + DateTime.Now.ToString("%H:mm:ss:ff") + ", " + dQuoteDate.ToString("%H:mm:ss") + ", " + EURUSDBid.ToString() + ", " + EURUSDAsk.ToString() + ", " + USDJPYBid.ToString() + ", " + USDJPYAsk.ToString() + ", " + EURJPYBid.ToString() + ", " + EURJPYAsk.ToString() + ", " + BBScpi.ToString() + ", " + SSBcpi.ToString());
                sw.Flush();
                sw.Close();
            }
            catch (System.Exception er)
            {
                //    System.Windows.Forms.MessageBox.Show("Logging error: " + er.Message);
            }
        }
        #endregion

    }

}