using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace GR
{

    public class SSBOrderClass
    {
        public void Reset()
        {
            _targetCPI = 0;
            _targetEURUSDBid = 0;
            _targetUSDJPYBid = 0;
            _targetEURJPYAsk = 0;
            _actualCPI = 0;
            _actualEURUSDBid = 0;
            _actualUSDJPYBid = 0;
            _actualEURJPYAsk = 0;
            TradeQTY = 0;
            TradeCompleted = false;
        }
        private double _targetCPI, _targetEURUSDBid, _targetUSDJPYBid, _targetEURJPYAsk;
        public double TargetCPI
        {
            get { return _targetCPI; }
        }
        public double TargetEURUSDBid
        {
            get { return _targetEURUSDBid; }
            set
            {
                _targetEURUSDBid = value;
                SetTargetCPI();
            }
        }
        public double TargetUSDJPYBid
        {
            get { return _targetUSDJPYBid; }
            set
            {
                _targetUSDJPYBid = value;
                SetTargetCPI();
            }
        }
        public double TargetEURJPYAsk
        {
            get { return _targetEURJPYAsk; }
            set
            {
                _targetEURJPYAsk = value;
                SetTargetCPI();
            }
        }
        private void SetTargetCPI()
        {
            if (_targetEURUSDBid != 0 && _targetUSDJPYBid != 0 && _targetEURJPYAsk != 0)
                _targetCPI = (_targetEURUSDBid * _targetUSDJPYBid * (1 / _targetEURJPYAsk)) * 100;
        }
        private double _actualCPI, _actualEURUSDBid, _actualUSDJPYBid, _actualEURJPYAsk;

        public double ActualCPI
        {
            get { return _actualCPI; }
        }
        public double ActualEURUSDBid
        {
            get { return _actualEURUSDBid; }
            set
            {
                _actualEURUSDBid = value;
                SetActualCPI();
            }
        }
        public double ActualUSDJPYBid
        {
            get { return _actualUSDJPYBid; }
            set
            {
                _actualUSDJPYBid = value;
                SetActualCPI();
            }
        }
        public double ActualEURJPYAsk
        {
            get { return _actualEURJPYAsk; }
            set
            {
                _actualEURJPYAsk = value;
                SetActualCPI();
            }
        }
        private void SetActualCPI()
        {
            if (_actualEURUSDBid != 0 && _actualUSDJPYBid != 0 && _actualEURJPYAsk != 0)
                _actualCPI = (_actualEURUSDBid * _actualUSDJPYBid * (1 / _actualEURJPYAsk)) * 100;
        }

        private int _tradeQTY;
        public int TradeQTY
        {
            get { return _tradeQTY; }
            set
            {
                _tradeQTY = value;
                //double qty because we want to detect whether the next
                //transaction is completed
                _EURUSDQTY = (Reverse) ? (-(value) * 2) : -(value);
                _USDJPYQTY = (Reverse) ? (-(value) * 2) : -(value);
                _EURJPYQTY = (Reverse) ? (-(value) * 2) : -(value);
            }
        }

        private int _EURUSDQTY, _USDJPYQTY, _EURJPYQTY;
        public int EURUSDQTY
        {
            get { return _EURUSDQTY; }
            set
            {
                _EURUSDQTY += value;
                CheckCompleted();
            }
        }
        public int USDJPYQTY
        {
            get { return _USDJPYQTY; }
            set
            {
                _USDJPYQTY += value;
                CheckCompleted();
            }
        }
        public int EURJPYQTY
        {
            get { return _EURJPYQTY; }
            set
            {
                _EURJPYQTY += value;
                CheckCompleted();
            }
        }
        private void CheckCompleted()
        {
            if (_EURUSDQTY == 0 && _USDJPYQTY == 0 && _EURJPYQTY == 0)
                TradeCompleted = true;
        }
        public bool Reverse = false;
        private bool _tradeCompleted = false;
        public bool TradeCompleted
        {
            get { return _tradeCompleted; }
            set
            {
                _tradeCompleted = value;
                if (value)
                    LogTrade();
            }
        }
        private void LogTrade()
        {
            try
            {
                string sPathName = System.Windows.Forms.Application.StartupPath.Remove(System.Windows.Forms.Application.StartupPath.IndexOf("bin")) + @"TradeLogs\";
                string sLogTime = DateTime.Now.ToShortDateString().ToString() + " " + DateTime.Now.ToLongTimeString().ToString();
                StreamWriter sw = new StreamWriter(sPathName + DateTime.Now.ToShortDateString().Replace('/','_') + ".txt", true);
                sw.WriteLine(sLogTime + ": SSB");
                sw.WriteLine("TargetCPI: " + _targetCPI.ToString() + ", TargetEURUSDBid: " + _targetEURUSDBid.ToString() + ", TargetUSDJPYBid: " + _targetUSDJPYBid.ToString() + ", TargetEURJPYAsk: " + _targetEURJPYAsk.ToString());
                sw.WriteLine("ActualCPI: " + _actualCPI.ToString() + ", ActualEURUSDBid: " + _actualEURUSDBid.ToString() + ", ActualUSDJPYBid: " + _actualUSDJPYBid.ToString() + ", ActualEURJPYAsk: " + _actualEURJPYAsk.ToString());
                sw.WriteLine("");
                sw.Flush();
                sw.Close();
            }
            catch (System.Exception er)
            {
            //    System.Windows.Forms.MessageBox.Show("Logging error: " + er.Message);
            }
        }
    }
}
